/* vim: set sw=8 ts=8 si : */
/*********************************************
* Author: Guido Socher
* Copyright: GPL V2
**********************************************/
#include <avr/io.h>
#include "config.h"


// return analog value of a given channel. Works without interrupt 
int convertanalog(unsigned char channel) 
{
	unsigned char adlow,adhigh;
	int result;
        /* enable analog to digital conversion in single run mode
        *  without noise canceler function. See datasheet of atmega88 page 250
        * We set ADPS2=1 ADPS0=1 ADPS1=1 to have a clock division factor of 128.
        * This is needed to stay in the recommended range of 50-200kHz 
	* Clock freq=8 or 12MHz
        * ADEN: Analog Digital Converter Enable
        * ADIE: ADC Interrupt Enable  (0=no interrupt)
        * ADIF: ADC Interrupt Flag
        * ADCSR: ADC Control and Status Register
        * ADPS2..ADPS0: ADC Prescaler Select Bits
	* REFS: Reference Selection Bits (page 203)
        */

	// int-ref with external capacitor at AREF pin: 
	// 1.10V int ref=REFS1=1,REFS0=1
	// write only the lower 3 bit for channel selection
	//
#ifdef USEAVCCREF
	// AVcc ref
	ADMUX=(1<<REFS0)|(channel & 0x0f);
#else
	// For the weather station we use the intenal 1.1V ref:
	ADMUX=(1<<REFS1)|(1<<REFS0)|(channel & 0x0f);
#endif
	// 
        ADCSRA=(1<<ADEN)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);
	// switch off digital input line:
	//DIDR0=(1<<channel)& 0x1f;

	//  start conversion 
	ADCSRA|= (1<<ADSC);
	while(bit_is_set(ADCSRA,ADSC)); // wait for result 
	adlow=ADCL; // read low first !! 
	adhigh=ADCH; 
	result=(adhigh<<8)|(adlow & 0xFF);
	// build avarage by doing a second cycle:
	ADCSRA|= (1<<ADSC);
	while(bit_is_set(ADCSRA,ADSC)); // wait for result 
	adlow=ADCL; // read low first !! 
	adhigh=ADCH; 
	result+=(adhigh<<8)|(adlow & 0xFF);
	return(result/2);
}


// We use integer arithmetics here. An int is 16bit. To have
// good accuracy the calculations need to be done such that
// the numbers are as large as possible but not out of range (less than +/- 32000)
// Always multiply first and then divide not the other way round.

// temperature sensor1, results in celsius times 10:
int tempcelsius0(int adcval){
	int i; //int is 16bit
	// write on several lines to ensure order:
	i=(adcval *10);
	i/=KELVINPERADC0;
	i=i*10 - OFFSET0;
	return(i);
}

// temperature sensor2, results in celsius times 10:
int tempcelsius1(int adcval){
	int i; // int is 16bit
	// write on several lines to ensure order:
	i=(adcval *10);
	i/=KELVINPERADC1;
	i=i*10 - OFFSET1;
	return(i);
}

// convert temperature to kelvin times 10 
int c2f(int celsiustimes10){
	return((celsiustimes10 * 18 + 3200)/10);
}

// air pressure in hPa *10
int pressure(int adcval){
	int i; // int is 16bit
	// we can not multiply by 100 as 16bit would overrun.
	// therefore first multiply by 25 and then by 4
	i=(adcval+(ERROR_A))*25/GAIN_A;
	i=i*4;
	return(i+REFTERM_A);
}

int pressureATsealevel(int phpa10){
	return(phpa10+SEALEVELOFFSET);
}
